/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.develop;

import java.util.List;
import java.util.Map;

import org.opengion.hayabusa.develop.AbstractJspCreate;
import org.opengion.hayabusa.develop.JspConvertEntity;
import org.opengion.fukurou.xml.OGElement;
import org.opengion.fukurou.xml.OGNode;
import org.opengion.fukurou.xml.OGNodeType;

/**
 * query.jspの &lt;og:hideMenu&gt;タグ 内の &lt;og:column&gt;タグを作成します。
 * column タグは、部分置換ではなく、hideMenu内の、table 部分からの全面置換です。（部分置換は難しかったので）
 * hideMenu は、通常の column タグの出力制限以上のカラムを書き出します。
 * 具体的には、TD_COUNT（初期値=3）＊ TR_COUNT（初期値=2）を超える検索条件の時のみ行います。
 * それ以下の場合は、hideMenu タグは書き出しません。
 * 
 * これと、JspCreate_COLUMN クラスは、密接に関連していますので、ご注意ください。
 *
 * ●使用例
 *      <table summary = "layout" >
 *          <tr><og:column ・・・ /> ・・・TD_COUNT（初期値=3）</tr>
 *                  ・・・・ TR_COUNT（初期値=2）
 *      </table>
 *   <og:hideMenu>
 *      <table summary = "layout" >
 *          <tr>
 *              <og:column
 *                  name       = column.getColumnName() 
 *                  defaultVal = column.getDefaultValue()
 *                  must       = "true"         ("1".equals( column.getMust() ))
 *                  clazz      = "aimai"        (ope.startsWith( "lk" ))
 *              />
 *              <og:column
 *                  ・・・・
 *              />
 *          </tr>
 *          <tr>
 *                  ・・・・
 *          </tr>
 *      </table>
 *   </og:hideMenu>
 *
 * @og.rev 5.6.4.4 (2013/05/31) 新規作成。hideMenu の対応
 *
 * @version  5.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK7.0,
 */
public class JspCreate_HIDEMENU extends JspCreate_COLUMN {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "5.6.4.4 (2013/05/31)" ;

	private List<JspConvertEntity> QUERY_ROWS ;
	private boolean IS_NULL ;

	/**
	 * 初期化メソッド
	 *
	 * 内部で使用する JspConvertEntity の リスト のマップを受け取り、初期化を行います。
	 *
	 * @param	master	JspConvertEntityのリストのマップ
	 */
	@Override
	protected void init( final Map<String,List<JspConvertEntity>> master ) {
		QUERY_ROWS = master.get("QUERY");
		IS_NULL = !isNotEmpty( QUERY_ROWS );
		KEY  = ":hideMenu";
		NAME = "query";
	}

	/**
	 * JSPに出力するタグの内容を作成します。
	 * 引数より作成前のタグの属性内容を確認するする事が出来ます。
	 *
	 * @param ele OGElementエレメントオブジェクト
	 * @param	nameSpace	このドキュメントのnameSpace( og とか mis とか )
	 *
	 * @return	変換された文字列
	 * @throws Throwable 変換時のエラー
	 */
	@Override
	protected String execute( final OGElement ele , final String nameSpace )  throws Throwable {
		if( IS_NULL ) { return ""; }

		if( QUERY_ROWS.size() <= TD_COUNT*TR_COUNT ) { return ""; }		// 指定以上のカラムがないと、hideMenu を作成しません。

		// 既存の設定値をすべて削除します。ホントは自動登録した分だけを削除すべき。
		OGElement newEle  = new OGElement( "og:hideMenu" );

		OGElement tblEle  = new OGElement( "table" );
		newEle.addNode( tblEle );

		OGElement tr = null;
			for( int i=(TD_COUNT*TR_COUNT); i<QUERY_ROWS.size(); i++ ) {
				JspConvertEntity column = QUERY_ROWS.get(i);
				if( i%TD_COUNT == 0 ) {
					tr = new OGElement( "tr" );
					tblEle.addNode( tr );
				}
				tr = trElement( tr,column );
			}

		return newEle.getText( 0 );
	}

	/**
	 * JSPに出力するタグの内容を作成します。
	 * 引数より作成前のタグの属性内容を確認するする事が出来ます。
	 *
	 * @og.rev 5.2.1.0 (2010/10/01) メソッドの引数を、OGAttributes から OGElement に変更します。
	 * @og.rev 5.2.1.0 (2010/10/01) 名前空間を、og 決め打ちから、引数を使用するように変更します。
	 * @og.rev 5.6.1.2 (2013/02/22) XML処理するように変更します。
	 *
	 * @param ele OGElementエレメントオブジェクト
	 * @param	nameSpace	このドキュメントのnameSpace( og とか mis とか )
	 *
	 * @return	変換された文字列
	 * @throws Throwable 変換時のエラー
	 */
//	@Override
//	protected String execute( final OGElement ele , final String nameSpace )  throws Throwable {
//		if( IS_NULL ) { return ""; }
//
//		String ns = (nameSpace.length() == 0) ? "" : nameSpace + ":" ;	// 5.2.1.0 (2010/10/01) 名前空間
//
//		// TODO Auto-generated method stub
//		//書き出す文字列を作成開始。
//		StringBuilder sbTub = new StringBuilder();
//
//		sbTub.append( "<table summary=\"layout\">" ).append( CR );
//		for( int i=0; i<QUERY_ROWS.size(); i++ ) {
//			JspConvertEntity column = QUERY_ROWS.get(i);
//			if( i%TD_COUNT == 0 ) {
//				if( i>0 ) { sbTub.append( "  </tr>" ).append( CR ); }
//				sbTub.append( "  <tr>" ).append( CR );
//			}
//			sbTub.append( "\t<" ).append( ns ).append( "column name=\"" );
//
//			sbTub.append( column.getColumnName() );
//			sbTub.append( "\"" );
//			if ( column.getDefaultValue() != null && column.getDefaultValue().trim().length() > 0) {
//				sbTub.append( " defaultVal=\"");
//				sbTub.append( column.getDefaultValue() );
//				sbTub.append( "\" ");
//			}
//			if ( "1".equals( column.getMust() )){
//				sbTub.append( " must=\"true\" ");
//			}
//			String ope = column.getRemarks();
//			if( ope != null && ope.startsWith( "lk" ) ) { sbTub.append( " clazz=\"aimai\" "); }
//
//			sbTub.append( " />" ).append( CR );
//		}
//		sbTub.append( "  </tr>" ).append( CR );
//		sbTub.append( "</table>" ).append( CR );
//
//		return sbTub.toString();
//	}
}
